using Nemerle.Collections;
using Nemerle.Compiler;
using Nemerle.Compiler.Parsetree;
using Nemerle.Internal;
using Nemerle.Macros;

namespace Oyster.Units.Macros
{
	/// Contains some operator-specific helper functionality
	module OpUtil
	{
		/// Used inside ariphmetic op macros
		public Body(
			ctx : Typer,
			lexpr : PExpr,
			rexpr : PExpr,
			opFun : PExpr * PExpr -> PExpr,
			bodyFun : TypeInfo * TypeInfo * double * double * list[int] * list[int] -> PExpr) : PExpr
		{
			match ((Util.GetExprType(ctx, lexpr), Util.GetExprType(ctx, rexpr)))
			{
				| (Some(ltype), Some(rtype)) =>
					match ((Util.ParseUnitSuffix(ltype), Util.ParseUnitSuffix(rtype)))
					{
						| (lpowers, rpowers) when lpowers.Length > 0 && rpowers.Length > 0 =>
							def getCoef(t) { Util.GetUnitNormCoef(t).Value };
							bodyFun(ltype, rtype, getCoef(ltype), getCoef(rtype), lpowers, rpowers)
						| _ => opFun(lexpr, rexpr)
					}
				| _ => opFun(lexpr, rexpr)
			}
		}

		/// Used inside + and - macros
		public AddSubBody(
			ctx : Typer,
			lexpr : PExpr,
			rexpr : PExpr,
			opName : string,
			opFun : PExpr * PExpr -> PExpr) : PExpr
		{
			Body(
				ctx,
				lexpr,
				rexpr,
				opFun,
				fun (ltype, _, lcoef, rcoef, lpowers, rpowers)
				{
					if (lpowers.Equals(rpowers))
					{
						def ctorParam = opFun(
							<[ $lexpr.Value ]>,
							<[ double.op_Multiply($rexpr.Value, $(rcoef / lcoef : double)) ]>);
						<[ $(Util.UseSiteExpr(ltype.FullName))($ctorParam) ]>
					}
					else
					{
						Message.FatalError(
							$"Expression { $lexpr [$opName] $rexpr } can't be compiled "
							 "because units are having different dimensions.")
					}
				})
		}

		/// Used inside * and / macros
		public MulDivBody(
			ctx : Typer,
			lexpr : PExpr,
			rexpr : PExpr,
			opFun : PExpr * PExpr -> PExpr,
			powersFun : int * int -> int,
			coefFun : double * double -> double) : PExpr
		{
			Body(
				ctx,
				lexpr,
				rexpr,
				opFun,
				fun (_, _, lcoef, rcoef, lpowers, rpowers)
				{
					def powers = List.Map2(lpowers, rpowers, powersFun);
					def t = TypeGenerator.GenerateUnitType(ctx.Env, "", 1.0, powers);
					def opExpr = opFun(<[ $lexpr.Value ]>, <[ $rexpr.Value ]>);
					<[ $(t.Name : usesite)(double.op_Multiply($opExpr, $(coefFun(lcoef, rcoef) : double))) ]>
				})
		}
	}


	/// Units addition macro-operator
	macro @+(lexpr, rexpr)
	{
		OpUtil.AddSubBody(ImplicitCTX(), lexpr, rexpr, "+", Util.AddExpr)
	}

	/// Units subtraction macro-operator
	macro @-(lexpr, rexpr)
	{
		OpUtil.AddSubBody(ImplicitCTX(), lexpr, rexpr, "-", Util.SubExpr)
	}

	/// Units multiply macro-operator
	macro @*(lexpr, rexpr)
	{
		OpUtil.MulDivBody(ImplicitCTX(), lexpr, rexpr, Util.MulExpr, _ + _, _ * _)
	}

	/// Units divide macro-operator
	macro @/(lexpr, rexpr)
	{
		OpUtil.MulDivBody(ImplicitCTX(), lexpr, rexpr, Util.DivExpr, _ - _, _ / _)
	}
}
