[assembly: Oyster.Units.Macros.UnitsDefinition([
	// SI
	basisUnits (Si)
	(
		Mass[ kg ],
		Length[ m ],
		Time[ sec ],
		Temperature[ K ],
		CurrentStrength[ A ],
		LightIntensity,
		QuantityOfSubstance
	),

	basisAliases (Si)
	(
		Area = (Length ^ 2),
		Volume = Area * Length,
		Velocity = Length / Time,
		Acceleration = Velocity / Time,
		AngularVelocity = Velocity,
		AngularAcceleration = Acceleration,
		Density = Mass / Volume,

		Force = Mass * Acceleration,
		SpecificGravity = Mass / (Length ^ 2) / (Time ^ 2),
		Pressure = SpecificGravity * Length,

		LinearMomentum = Mass * Velocity,
		MomentOfInertia = Mass * Area,

		Energy = Force * Length,
		Power = Energy / Time,

		DynamicViscosity = Mass / Length / Time,
		KinematicViscosity = DynamicViscosity,

		HeatCapacity = Energy / Temperature,
		Entropy = HeatCapacity,
		SpecificHeat = HeatCapacity / Mass,
		SpecificEntropy = SpecificHeat,

		Charge = Time * CurrentStrength,
		LinearDensityOfCharge = Charge / Length,
		SurfaceDensityOfCharge = Charge / Area,
		ElectricFluxDensity = SurfaceDensityOfCharge,
		PackedDensityOfCharge = Charge / Volume,

		Voltage = Power / CurrentStrength,
		ElectricForce = Force / CurrentStrength,
		Resistance = Voltage / CurrentStrength,
		SpecificResistance = Resistance * Length,

		Permittance = Time / Resistance,

		CurrentDensity = CurrentStrength / (Length ^ 2)
	),

	// CGS
	units (Cgs)
	(
		Mass[ g ] - 1.0 / 1000.0,
		Length[ cm ] - 1.0 / 100.0,
		Time,
		Temperature,
		CurrentStrength - 1.0 / 3000000000.0,
		LightIntensity,
		QuantityOfSubstance
	),

	// Other units
	units (_)
	(
		HorsePower = Power - 1471.0 / 2.0
	)
])]
